window.Gleam.initSearch([{"doc":"nibble","title":"nibble","content":"# nibble\n\nA lexer and parser combinator library inspired by [`elm/parser`](https://github.com/elm/parser).\n\n[![Package Version](https://img.shields.io/hexpm/v/nibble)](https://hex.pm/packages/nibble)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/nibble/)\n\n✨ This project is written in **pure Gleam** so you can use it anywhere Gleam\nruns: Erlang, Elixir, Node, Deno, and the browser!\n\n## Quick start\n\nIf you just want to get a feel for what nibble can do, check out the example\nbelow.\n\n```gleam\nimport gleam/option.{Some, None}\nimport nibble.{do, return}\nimport nibbler/lexer\n\ntype Point {\n  Point(x: Int, y: Int)\n}\n\ntype Token {\n  Num(Int)\n  LParen\n  RParen\n  Comma\n}\n\npub fn main() {\n  // Your lexer knows how to take an input string and\n  // turn it into a flat list of tokens. You define the\n  // type of token you want to use, but nibble will wrap\n  // that up in its own `Token` type that includes the\n  // source span and original lexeme for each token.\n  let lexer = lexer.simple([\n    lexer.int(Num),\n    lexer.token(&quot;(&quot;, LParen),\n    lexer.token(&quot;)&quot;, RParen),\n    lexer.token(&quot;,&quot;, Comma),\n    // Skip over whitespace, we don&#39;t care about it!\n    lexer.whitespace(Nil)\n    |&gt; lexer.ignore\n  ])\n\n  // Your parser(s!) know how to transform a list of\n  // tokens into whatever you want. You have the full\n  // power of Gleam here, so you can go wild!\n  let parser = {\n    use _ &lt;- do(nibble.token(LParen))\n    use x &lt;- do(int_parser())\n    use _ &lt;- do(nibble.token(Comma))\n    use y &lt;- do(int_parser())\n    use _ &lt;- do(nibble.token(RParen))\n\n    return(Point(x, y))\n  }\n\n  let int_parser = {\n    // Use `take_map` to only consume certain kinds of tokens and transform the\n    // result.\n    use tok &lt;- nibble.take_map\n\n    case tok {\n      Num(n) -&gt; Some(n)\n      _ -&gt; None\n    }\n  }\n\n  let assert Ok(tokens) = lexer.run(&quot;(1, 2)&quot;, lexer)\n  let assert Ok(point) = nibble.run(tokens, parser)\n\n  point.x //=&gt; 1\n  point.y //=&gt; 2\n}\n```\n\n## Installation\n\nIf available on Hex this package can be added to your Gleam project:\n\n```sh\ngleam add nibble\n```\n\nand its documentation can be found at &lt;https://hexdocs.pm/nibble&gt;.\n","url":"index.html"},{"doc":"nibble","title":"nibble","content":"","url":"intro.html"},{"doc":"nibble","title":"nibble","content":"","url":"lexing.html"},{"doc":"nibble","title":"nibble","content":"","url":"parsing.html"},{"doc":"nibble","title":"nibble","content":"","url":"parse-contexts.html"},{"doc":"nibble","title":"nibble","content":"","url":"backtracking.html"},{"doc":"nibble","title":"nibble","content":"# Lexer Modes\n\nUp until now we have been running our lexer using `lexer.simple`. As the name\nimplies, this is the simplest way to use Nibble&#39;s lexer and it is context-free.\nWhere possible we should try to stick to these simple lexers, but sometimes we\nneed to be able to lex things that are context-sensitive. That&#39;s where lexer\nmodes come in!\n\n## Indentation Sensitivity\n\nLet&#39;s imagine we&#39;re writing a lexer for a Python-ish programming language and\nwe want to produce `Indent` and `Dedent` tokens to represent indentation. We\nmight define our tokens like this:\n\n```gleam\npub type TokenT {\n  Var(String)\n  Str(String)\n  Num(Int)\n\n  // Keywords\n  Def\n  For\n  In\n  Print\n\n  // Indentation\n  Indent(Int)\n  Dedent(Int)\n}\n```\n\nWe could represent a chunk of code like this:\n\n```python\ndef wibble arr\n  for x in arr\n    print x\n\n  print &quot;done!&quot;\n\ndef wobble\n  wibble [1, 2, 3]\n```\n\nIndentation would change the meaning of this program, so we need to know when we\nare inside a block of indented code or not. Our `Indent` and `Dedent` tokens\ncarry with them the level of indentation they represent such that when we come to\nparsing we can make sure everything is valid, but how do we produce the tokens in\nthe first place?\n\nWe&#39;ll need to do two things: (1) write a custom matcher using `lexer.custom` and\n(2) store the current indentation level as the lexer&#39;s mode.\n\n```gleam\npub opaque type Lexer(a, mode)\npub opaque type Matcher(a, mode)\n```\n\nModes allow us to chose different matchers for different contexts, or inject\nstate into our matchers. For our indentation-sensitive lexer, that means we&#39;ll\nend up with `Lexer` and `Matcher` types like this:\n\n```gleam\ntype Lexer = nibble.Lexer(TokenT, Int)\ntype Matcher = nibble.Matcher(TokenT, Int)\n```\n\nTo write our `indentation` matcher, we&#39;ll count the number of spaces that immediately\nfollow a newline and compare that to the current indentation level. If that number\nis less than the current indentation level, we&#39;ll produce a `Dedent` token, otherwise\nwe&#39;ll produce an `Indent` token. In either case we&#39;ll also update the lexer&#39;s\nmode with the new indentation level for subsequent lines.\n\n```gleam\nfn indentation() -&gt; Matcher(TokenT, Int) {\n  let assert Ok(is_indent) = regex.from_string(&quot;^\\\\n[ \\\\t]*&quot;)\n  use current_indent, lexeme, lookahead &lt;- lexer.custom\n\n  case regex.check(is_indent, lexeme), lookahead {\n    False, _ -&gt; NoMatch\n    True, &quot; &quot; | True, &quot;\\t&quot; -&gt; Skip\n    True, &quot;\\n&quot; -&gt; Drop(current_indent)\n    True, _ -&gt; {\n      let spaces = string.length(lexeme) - 1\n\n      case int.compare(spaces, current_indent) {\n        Lt -&gt; Keep(Dedent(spaces), spaces)\n        Eq if spaces == 0 -&gt; Drop(0)\n        Eq -&gt; Keep(Indent(spaces), spaces)\n        Gt -&gt; Keep(Indent(spaces), spaces)\n      }\n    }\n  }\n}\n```\n\nThere&#39;s actually a little more going on here than I just described, so let&#39;s\nbreak the pattern matching down case by case.\n\n```gleam\nFalse, _ -&gt; NoMatch\n```\n\n- This branch should be self-explanatory. If the lexeme doesn&#39;t match the regex\n  we have for indentation then we don&#39;t have a match and should tell the lexer to\n  move on and attempt a different matcher.\n\n```gleam\nTrue, &quot; &quot; | True, &quot;\\t&quot; -&gt; Skip\n```\n\n- If the lexeme matches the indentation regex but we can see from the lookahead\n  that it is followed by more indentation, then we don&#39;t want to produce a token\n  right now. We _also_ don&#39;t want the lexer to try any other matchers that might\n  consume these lexeme (like `lexer.whitespace`) so we tell the lexer to _skip_\n  checking any other matchers and move on to the next lexeme.\n\n```gleam\nTrue, &quot;\\n&quot; -&gt; Drop(current_indent)\n```\n\n- When the lexeme matches the indentation regex but it&#39;s followed by a new line\n  that means we have an empty line of just whitespace. You could choose to emit a\n  token here if you wanted to, but in this case we&#39;ll ignore empty lines entirely\n  and instruct the lexer to _drop_ the lexeme without producing anything. Importantly\n  we preserve the current indentation level as the lexer&#39;s mode going forward.\n\n```gleam\nTrue, _ -&gt; {\n  let spaces = string.length(lexeme) - 1\n\n  case int.compare(spaces, current_indent) {\n    Lt -&gt; Keep(Dedent(spaces), spaces)\n    Eq if spaces == 0 -&gt; Drop(0)\n    Eq -&gt; Keep(Indent(spaces), spaces)\n    Gt -&gt; Keep(Indent(spaces), spaces)\n  }\n}\n```\n\n- This is the actual indentation logic; we hit this branch when the lookahead is\n  any other grapheme. To get the new indentation level we just need to count the\n  length of the string, remembering to subtract one for the leading newline. By\n  comparing the new indentation to the current, we can work out what token to keep.\n\n  - If the new indentation is less than the current, we keep a `Dedent` token and\n    update the lexer&#39;s mode to the new indentation level.\n  - If the new indentation is is equal to the current _but_ that level is zero,\n    then we drop the lexeme without producing any indentation token at all.\n  - Otherwise, if the new indentation is equal or greater than the current, we\n    keep an `Indent` token and update the lexer&#39;s mode to the new indentation level.\n\n## String Interpolation\n","url":"lexer-modes.html"},{"doc":"nibble","title":"nibble","content":"","url":"pratt-parsing.html"},{"doc":"nibble/lexer","title":"Error","content":"pub type Error {\n  NoMatchFound(row: Int, col: Int, lexeme: String)\n}\n\n\nNoMatchFound(row: Int, col: Int, lexeme: String)\n\n\nSynonyms:\nnibble/lexer.Error\nnibble/lexer Error","url":"nibble/lexer.html#Error"},{"doc":"nibble/lexer","title":"Lexer","content":"pub opaque type Lexer(a, mode)\n\n\n\nSynonyms:\nnibble/lexer.Lexer\nnibble/lexer Lexer","url":"nibble/lexer.html#Lexer"},{"doc":"nibble/lexer","title":"Match","content":"pub type Match(a, mode) {\n  Keep(a, mode)\n  Skip\n  Drop(mode)\n  NoMatch\n}\n\nKeep(a, mode)\n\n\nSkip\n\n\nDrop(mode)\n\n\nNoMatch\n\n\nSynonyms:\nnibble/lexer.Match\nnibble/lexer Match","url":"nibble/lexer.html#Match"},{"doc":"nibble/lexer","title":"Matcher","content":"pub opaque type Matcher(a, mode)\n\n\n\nSynonyms:\nnibble/lexer.Matcher\nnibble/lexer Matcher","url":"nibble/lexer.html#Matcher"},{"doc":"nibble/lexer","title":"Span","content":"pub type Span {\n  Span(\n    row_start: Int,\n    col_start: Int,\n    row_end: Int,\n    col_end: Int,\n  )\n}\n\n\nSpan(row_start: Int, col_start: Int, row_end: Int, col_end: Int)\n\n\nSynonyms:\nnibble/lexer.Span\nnibble/lexer Span","url":"nibble/lexer.html#Span"},{"doc":"nibble/lexer","title":"Token","content":"pub type Token(a) {\n  Token(span: Span, lexeme: String, value: a)\n}\n\n\nToken(span: Span, lexeme: String, value: a)\n\n\nSynonyms:\nnibble/lexer.Token\nnibble/lexer Token","url":"nibble/lexer.html#Token"},{"doc":"nibble/lexer","title":"advanced","content":"pub fn advanced(\n  matchers: fn(a) -&gt; List(Matcher(b, a)),\n) -&gt; Lexer(b, a)\n\n\nSynonyms:\nnibble/lexer.advanced\nnibble/lexer advanced","url":"nibble/lexer.html#advanced"},{"doc":"nibble/lexer","title":"comment","content":"pub fn comment(\n  start: String,\n  to_value: fn(String) -&gt; a,\n) -&gt; Matcher(a, b)\n\nSynonyms:\nnibble/lexer.comment\nnibble/lexer comment","url":"nibble/lexer.html#comment"},{"doc":"nibble/lexer","title":"custom","content":"pub fn custom(\n  f: fn(a, String, String) -&gt; Match(b, a),\n) -&gt; Matcher(b, a)\n\n\nSynonyms:\nnibble/lexer.custom\nnibble/lexer custom","url":"nibble/lexer.html#custom"},{"doc":"nibble/lexer","title":"drop","content":"pub fn drop(f: fn(String, String) -&gt; Bool) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.drop\nnibble/lexer drop","url":"nibble/lexer.html#drop"},{"doc":"nibble/lexer","title":"float","content":"pub fn float(to_value: fn(Float) -&gt; a) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.float\nnibble/lexer float","url":"nibble/lexer.html#float"},{"doc":"nibble/lexer","title":"float_with_separator","content":"pub fn float_with_separator(\n  separator: String,\n  to_value: fn(Float) -&gt; a,\n) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.float_with_separator\nnibble/lexer float_with_separator","url":"nibble/lexer.html#float_with_separator"},{"doc":"nibble/lexer","title":"identifier","content":"pub fn identifier(\n  start: String,\n  inner: String,\n  reserved: Set(String),\n  to_value: fn(String) -&gt; a,\n) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.identifier\nnibble/lexer identifier","url":"nibble/lexer.html#identifier"},{"doc":"nibble/lexer","title":"ignore","content":"pub fn ignore(matcher: Matcher(a, b)) -&gt; Matcher(c, b)\n\n\nSynonyms:\nnibble/lexer.ignore\nnibble/lexer ignore","url":"nibble/lexer.html#ignore"},{"doc":"nibble/lexer","title":"int","content":"pub fn int(to_value: fn(Int) -&gt; a) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.int\nnibble/lexer int","url":"nibble/lexer.html#int"},{"doc":"nibble/lexer","title":"int_with_separator","content":"pub fn int_with_separator(\n  separator: String,\n  to_value: fn(Int) -&gt; a,\n) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.int_with_separator\nnibble/lexer int_with_separator","url":"nibble/lexer.html#int_with_separator"},{"doc":"nibble/lexer","title":"into","content":"pub fn into(\n  matcher: Matcher(a, b),\n  f: fn(b) -&gt; b,\n) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.into\nnibble/lexer into","url":"nibble/lexer.html#into"},{"doc":"nibble/lexer","title":"keep","content":"pub fn keep(\n  f: fn(String, String) -&gt; Result(a, Nil),\n) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.keep\nnibble/lexer keep","url":"nibble/lexer.html#keep"},{"doc":"nibble/lexer","title":"keyword","content":"pub fn keyword(\n  str: String,\n  breaker: String,\n  value: a,\n) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.keyword\nnibble/lexer keyword","url":"nibble/lexer.html#keyword"},{"doc":"nibble/lexer","title":"map","content":"pub fn map(\n  matcher: Matcher(a, b),\n  f: fn(a) -&gt; c,\n) -&gt; Matcher(c, b)\n\n\nSynonyms:\nnibble/lexer.map\nnibble/lexer map","url":"nibble/lexer.html#map"},{"doc":"nibble/lexer","title":"number","content":"pub fn number(\n  from_int: fn(Int) -&gt; a,\n  from_float: fn(Float) -&gt; a,\n) -&gt; Matcher(a, b)\n\nSynonyms:\nnibble/lexer.number\nnibble/lexer number","url":"nibble/lexer.html#number"},{"doc":"nibble/lexer","title":"number_with_separator","content":"pub fn number_with_separator(\n  separator: String,\n  from_int: fn(Int) -&gt; a,\n  from_float: fn(Float) -&gt; a,\n) -&gt; Matcher(a, b)\n\nSynonyms:\nnibble/lexer.number_with_separator\nnibble/lexer number_with_separator","url":"nibble/lexer.html#number_with_separator"},{"doc":"nibble/lexer","title":"run","content":"pub fn run(\n  source: String,\n  lexer: Lexer(a, Nil),\n) -&gt; Result(List(Token(a)), Error)\n\n\nSynonyms:\nnibble/lexer.run\nnibble/lexer run","url":"nibble/lexer.html#run"},{"doc":"nibble/lexer","title":"run_advanced","content":"pub fn run_advanced(\n  source: String,\n  mode: a,\n  lexer: Lexer(b, a),\n) -&gt; Result(List(Token(b)), Error)\n\n\nSynonyms:\nnibble/lexer.run_advanced\nnibble/lexer run_advanced","url":"nibble/lexer.html#run_advanced"},{"doc":"nibble/lexer","title":"simple","content":"pub fn simple(matchers: List(Matcher(a, Nil))) -&gt; Lexer(a, Nil)\n\n\nSynonyms:\nnibble/lexer.simple\nnibble/lexer simple","url":"nibble/lexer.html#simple"},{"doc":"nibble/lexer","title":"skip","content":"pub fn skip(matcher: Matcher(a, b)) -&gt; Matcher(c, b)\n\n\nSynonyms:\nnibble/lexer.skip\nnibble/lexer skip","url":"nibble/lexer.html#skip"},{"doc":"nibble/lexer","title":"spaces","content":"pub fn spaces(token: a) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.spaces\nnibble/lexer spaces","url":"nibble/lexer.html#spaces"},{"doc":"nibble/lexer","title":"spaces_","content":"pub fn spaces_(to_value: fn(String) -&gt; a) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.spaces_\nnibble/lexer spaces_","url":"nibble/lexer.html#spaces_"},{"doc":"nibble/lexer","title":"string","content":"pub fn string(\n  char: String,\n  to_value: fn(String) -&gt; a,\n) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.string\nnibble/lexer string","url":"nibble/lexer.html#string"},{"doc":"nibble/lexer","title":"symbol","content":"pub fn symbol(\n  str: String,\n  breaker: String,\n  value: a,\n) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.symbol\nnibble/lexer symbol","url":"nibble/lexer.html#symbol"},{"doc":"nibble/lexer","title":"then","content":"pub fn then(\n  matcher: Matcher(a, b),\n  f: fn(a) -&gt; Match(c, b),\n) -&gt; Matcher(c, b)\n\n\nSynonyms:\nnibble/lexer.then\nnibble/lexer then","url":"nibble/lexer.html#then"},{"doc":"nibble/lexer","title":"token","content":"pub fn token(str: String, value: a) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.token\nnibble/lexer token","url":"nibble/lexer.html#token"},{"doc":"nibble/lexer","title":"try_identifier","content":"pub fn try_identifier(\n  start: String,\n  inner: String,\n  reserved: Set(String),\n  to_value: fn(String) -&gt; a,\n) -&gt; Result(Matcher(a, b), CompileError)\n\n\nSynonyms:\nnibble/lexer.try_identifier\nnibble/lexer try_identifier","url":"nibble/lexer.html#try_identifier"},{"doc":"nibble/lexer","title":"variable","content":"pub fn variable(\n  reserved: Set(String),\n  to_value: fn(String) -&gt; a,\n) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.variable\nnibble/lexer variable","url":"nibble/lexer.html#variable"},{"doc":"nibble/lexer","title":"whitespace","content":"pub fn whitespace(token: a) -&gt; Matcher(a, b)\n\n\nSynonyms:\nnibble/lexer.whitespace\nnibble/lexer whitespace","url":"nibble/lexer.html#whitespace"},{"doc":"nibble/lexer","title":"nibble/lexer","content":"","url":"nibble/lexer.html"},{"doc":"nibble","title":"DeadEnd","content":"pub type DeadEnd(tok, ctx) {\n  DeadEnd(\n    pos: Span,\n    problem: Error(tok),\n    context: List(#(Span, ctx)),\n  )\n}\n A dead end represents a the point where a parser that had committed down a\n path failed. It contains the position of the failure, the [`Error`](#Error)\n describing the failure, and the context stack for any parsers that had run.\n\nDeadEnd(\n  pos: Span,\n  problem: Error(tok),\n  context: List(#(Span, ctx)),\n)\n\n\nSynonyms:\nnibble.DeadEnd\nnibble DeadEnd","url":"nibble.html#DeadEnd"},{"doc":"nibble","title":"Error","content":"pub type Error(tok) {\n  BadParser(String)\n  Custom(String)\n  EndOfInput\n  Expected(String, got: tok)\n  Unexpected(tok)\n}\n\n\n\n\nBadParser(String)\n\n\nCustom(String)\n\n\nEndOfInput\n\n\nExpected(String, got: tok)\n\n\nUnexpected(tok)\n\n\nSynonyms:\nnibble.Error\nnibble Error","url":"nibble.html#Error"},{"doc":"nibble","title":"Loop","content":"pub type Loop(a, state) {\n  Continue(state)\n  Break(a)\n}\n\n\nContinue(state)\n\n\nBreak(a)\n\n\nSynonyms:\nnibble.Loop\nnibble Loop","url":"nibble.html#Loop"},{"doc":"nibble","title":"Parser","content":"pub opaque type Parser(a, tok, ctx)\n The `Parser` type has three paramteres, let&#39;s take a look at each of them:\n\n \n Parser(a, tok, ctx)\n // (1) ^\n // (2)    ^^^\n // (3)         ^^^\n \n\n 1) `a` is the type of value that the parser knows how to produce. If you were\n   writing a parser for a programming language, this might be your expression\n   type.\n\n 2) `tok` is the type of tokens that the parser knows how to consume. You can\n   take a look at the [`Token`](./nibble/lexer#Token) type for a bit more info,\n   but note that it&#39;s not necessary for the token stream to come from nibble&#39;s\n   lexer.\n\n 3) `ctx` is used to make error reporting nicer. You can place a parser into a\n   custom context. When the parser runs the context gets pushed into a stack.\n   If the parser fails you can see the context stack in the error message,\n   which can make error reporting and debugging much easier!\n\n\nSynonyms:\nnibble.Parser\nnibble Parser","url":"nibble.html#Parser"},{"doc":"nibble","title":"any","content":"pub fn any() -&gt; Parser(a, a, b)\n\n\nSynonyms:\nnibble.any\nnibble any","url":"nibble.html#any"},{"doc":"nibble","title":"backtrackable","content":"pub fn backtrackable(parser: Parser(a, b, c)) -&gt; Parser(a, b, c)\n By default, parsers will not backtrack if they fail after consuming at least\n one token. Passing a parser to `backtrackable` will change this behaviour and\n allows us to jump back to the state of the parser before it consumed any input\n and try another one.\n\n This is most useful when you want to quickly try a few different parsers using\n [`one_of`](#one_of).\n\n 🚨 Backtracing parsers can drastically reduce performance, so you should avoid\n them where possible. A common reason folks reach for backtracking is when they\n want to try multiple branches that start with the same token or same sequence\n of tokens.\n\n To avoid backtracking in these cases, you can create an intermediate parser\n that consumes the common tokens _and then_ use [`one_of`](#one_of) to try\n the different branches.\n\nSynonyms:\nnibble.backtrackable\nnibble backtrackable","url":"nibble.html#backtrackable"},{"doc":"nibble","title":"do","content":"pub fn do(\n  parser: Parser(a, b, c),\n  f: fn(a) -&gt; Parser(d, b, c),\n) -&gt; Parser(d, b, c)\n\n\nSynonyms:\nnibble.do\nnibble do","url":"nibble.html#do"},{"doc":"nibble","title":"do_in","content":"pub fn do_in(\n  context: a,\n  parser: Parser(b, c, a),\n  f: fn(b) -&gt; Parser(d, c, a),\n) -&gt; Parser(d, c, a)\n\n\nSynonyms:\nnibble.do_in\nnibble do_in","url":"nibble.html#do_in"},{"doc":"nibble","title":"eof","content":"pub fn eof() -&gt; Parser(Nil, a, b)\n\n\nSynonyms:\nnibble.eof\nnibble eof","url":"nibble.html#eof"},{"doc":"nibble","title":"fail","content":"pub fn fail(message: String) -&gt; Parser(a, b, c)\n Create a parser that consumes no tokens and always fails with the given\n error message.\n\nSynonyms:\nnibble.fail\nnibble fail","url":"nibble.html#fail"},{"doc":"nibble","title":"guard","content":"pub fn guard(cond: Bool, expecting: String) -&gt; Parser(Nil, a, b)\n\n\nSynonyms:\nnibble.guard\nnibble guard","url":"nibble.html#guard"},{"doc":"nibble","title":"in","content":"pub fn in(parser: Parser(a, b, c), context: c) -&gt; Parser(a, b, c)\n\n\nSynonyms:\nnibble.in\nnibble in","url":"nibble.html#in"},{"doc":"nibble","title":"inspect","content":"pub fn inspect(\n  parser: Parser(a, b, c),\n  message: String,\n) -&gt; Parser(a, b, c)\n Run the given parser and then inspect it&#39;s state.\nSynonyms:\nnibble.inspect\nnibble inspect","url":"nibble.html#inspect"},{"doc":"nibble","title":"lazy","content":"pub fn lazy(parser: fn() -&gt; Parser(a, b, c)) -&gt; Parser(a, b, c)\n Defer the creation of a parser until it is needed. This is often most useful\n when creating a parser that is recursive and is *not* a function.\n\nSynonyms:\nnibble.lazy\nnibble lazy","url":"nibble.html#lazy"},{"doc":"nibble","title":"loop","content":"pub fn loop(\n  init: a,\n  step: fn(a) -&gt; Parser(Loop(b, a), c, d),\n) -&gt; Parser(b, c, d)\n\n\nSynonyms:\nnibble.loop\nnibble loop","url":"nibble.html#loop"},{"doc":"nibble","title":"many","content":"pub fn many(parser: Parser(a, b, c)) -&gt; Parser(List(a), b, c)\n\n\nSynonyms:\nnibble.many\nnibble many","url":"nibble.html#many"},{"doc":"nibble","title":"many1","content":"pub fn many1(parser: Parser(a, b, c)) -&gt; Parser(List(a), b, c)\n\n\nSynonyms:\nnibble.many1\nnibble many1","url":"nibble.html#many1"},{"doc":"nibble","title":"map","content":"pub fn map(\n  parser: Parser(a, b, c),\n  f: fn(a) -&gt; d,\n) -&gt; Parser(d, b, c)\n\n\nSynonyms:\nnibble.map\nnibble map","url":"nibble.html#map"},{"doc":"nibble","title":"one_of","content":"pub fn one_of(parsers: List(Parser(a, b, c))) -&gt; Parser(a, b, c)\n\n\nSynonyms:\nnibble.one_of\nnibble one_of","url":"nibble.html#one_of"},{"doc":"nibble","title":"optional","content":"pub fn optional(\n  parser: Parser(a, b, c),\n) -&gt; Parser(Option(a), b, c)\n Try the given parser, but if it fails return\n [`None`](#https://hexdocs.pm/gleam_stdlib/gleam/option.html#Option) instead\n of failing.\n\nSynonyms:\nnibble.optional\nnibble optional","url":"nibble.html#optional"},{"doc":"nibble","title":"or","content":"pub fn or(parser: Parser(a, b, c), default: a) -&gt; Parser(a, b, c)\n Try the given parser, but if it fails return the given default value instead\n of failing.\n\nSynonyms:\nnibble.or\nnibble or","url":"nibble.html#or"},{"doc":"nibble","title":"replace","content":"pub fn replace(\n  parser: Parser(a, b, c),\n  with b: d,\n) -&gt; Parser(d, b, c)\n\n\nSynonyms:\nnibble.replace\nnibble replace","url":"nibble.html#replace"},{"doc":"nibble","title":"return","content":"pub fn return(value: a) -&gt; Parser(a, b, c)\n The simplest kind of parser. [`return`](#return) consumes no tokens and always\n produces the given value. Sometimes called [`succeed`](#succeed) instead.\n\n This function might seem useless at first, but it is very useful when used in\n combination with [`do`](#do) or [`then`](#then).\n\n \n import nibble.{do, return}\n\n fn unit8_parser() {\n   use int &lt;- do(int_parser())\n\n   case int &gt;= 0, int &lt;= 255 {\n     True, True -&gt;\n       return(int)\n\n     False, _ -&gt;\n       throw(&quot;Expected an int &gt;= 0&quot;)\n\n     _, False -&gt;\n       throw(&quot;Expected an int &lt;= 255&quot;)\n  }\n }\n \n\n 💡 [`return`](#return`) and [`succeed`](#succeed) are names for the same thing.\n We suggesting using `return` unqualified when using `do` and Gleam&#39;s `use`\n syntax, and `nibble.succeed` in a pipeline with `nibble.then`.\nSynonyms:\nnibble.return\nnibble return","url":"nibble.html#return"},{"doc":"nibble","title":"run","content":"pub fn run(\n  src: List(Token(a)),\n  parser: Parser(b, a, c),\n) -&gt; Result(b, List(DeadEnd(a, c)))\n Parsers don&#39;t do anything until they&#39;re run! The `run` function takes a\n [`Parser`](#Parser) and a list of [`Token`](./nibble/lexer#Token)s and\n runs it; returning either the parsed value or a list of [`DeadEnds`](#DeadEnd)\n where the parser failed.\n\nSynonyms:\nnibble.run\nnibble run","url":"nibble.html#run"},{"doc":"nibble","title":"sequence","content":"pub fn sequence(\n  parser: Parser(a, b, c),\n  separator sep: Parser(d, b, c),\n) -&gt; Parser(List(a), b, c)\n\n\nSynonyms:\nnibble.sequence\nnibble sequence","url":"nibble.html#sequence"},{"doc":"nibble","title":"span","content":"pub fn span() -&gt; Parser(Span, a, b)\n A parser that returns the current token position.\n\nSynonyms:\nnibble.span\nnibble span","url":"nibble.html#span"},{"doc":"nibble","title":"succeed","content":"pub fn succeed(value: a) -&gt; Parser(a, b, c)\n The simplest kind of parser. [`succeed`](#succeed) consumes no tokens and always\n produces the given value. Sometimes called [`return`](#return) instead.\n\n This function might seem useless at first, but it is very useful when used in\n combination with [`do`](#do) or [`then`](#then).\n\n \n import nibble\n\n fn unit8_parser() {\n   int_parser()\n   |&gt; nibble.then(fn(int) {\n     case int &gt;= 0, int &lt;= 255 {\n       True, True -&gt; succeed(int)\n       False, _ -&gt; fail(&quot;Expected an int &gt;= 0&quot;)\n       _, False -&gt; fail(&quot;Expected an int &lt;= 255&quot;)\n     }\n   })\n }\n \n\n 💡 [`succeed`](#succeed) and [`return`](#return) are names for the same thing.\n We suggest using `succeed` in a pipeline with `nibble.then`, and `return`\n unqalified when using `do` with Gleam&#39;s `use` syntax.\n\nSynonyms:\nnibble.succeed\nnibble succeed","url":"nibble.html#succeed"},{"doc":"nibble","title":"take_at_least","content":"pub fn take_at_least(\n  parser: Parser(a, b, c),\n  count: Int,\n) -&gt; Parser(List(a), b, c)\n\n\nSynonyms:\nnibble.take_at_least\nnibble take_at_least","url":"nibble.html#take_at_least"},{"doc":"nibble","title":"take_exactly","content":"pub fn take_exactly(\n  parser: Parser(a, b, c),\n  count: Int,\n) -&gt; Parser(List(a), b, c)\n\n\nSynonyms:\nnibble.take_exactly\nnibble take_exactly","url":"nibble.html#take_exactly"},{"doc":"nibble","title":"take_if","content":"pub fn take_if(\n  expecting: String,\n  predicate: fn(a) -&gt; Bool,\n) -&gt; Parser(a, a, b)\n\n\nSynonyms:\nnibble.take_if\nnibble take_if","url":"nibble.html#take_if"},{"doc":"nibble","title":"take_map","content":"pub fn take_map(\n  expecting: String,\n  f: fn(a) -&gt; Option(b),\n) -&gt; Parser(b, a, c)\n Take the next token and attempt to transform it with the given function. This\n is useful when creating reusable primtive parsers for your own tokens such as\n `take_identifier` or `take_number`.\n\nSynonyms:\nnibble.take_map\nnibble take_map","url":"nibble.html#take_map"},{"doc":"nibble","title":"take_map_while","content":"pub fn take_map_while(\n  f: fn(a) -&gt; Option(b),\n) -&gt; Parser(List(b), a, c)\n\n\nSynonyms:\nnibble.take_map_while\nnibble take_map_while","url":"nibble.html#take_map_while"},{"doc":"nibble","title":"take_map_while1","content":"pub fn take_map_while1(\n  expecting: String,\n  f: fn(a) -&gt; Option(b),\n) -&gt; Parser(List(b), a, c)\n\n\n 💡 If this parser succeeds, the list produced is guaranteed to be non-empty.\n Feel free to `let assert` the result!\n\nSynonyms:\nnibble.take_map_while1\nnibble take_map_while1","url":"nibble.html#take_map_while1"},{"doc":"nibble","title":"take_until","content":"pub fn take_until(\n  predicate: fn(a) -&gt; Bool,\n) -&gt; Parser(List(a), a, b)\n\n\nSynonyms:\nnibble.take_until\nnibble take_until","url":"nibble.html#take_until"},{"doc":"nibble","title":"take_until1","content":"pub fn take_until1(\n  expecting: String,\n  predicate: fn(a) -&gt; Bool,\n) -&gt; Parser(List(a), a, b)\n\n\n 💡 If this parser succeeds, the list produced is guaranteed to be non-empty.\n Feel free to `let assert` the result!\n\nSynonyms:\nnibble.take_until1\nnibble take_until1","url":"nibble.html#take_until1"},{"doc":"nibble","title":"take_up_to","content":"pub fn take_up_to(\n  parser: Parser(a, b, c),\n  count: Int,\n) -&gt; Parser(List(a), b, c)\n\n\nSynonyms:\nnibble.take_up_to\nnibble take_up_to","url":"nibble.html#take_up_to"},{"doc":"nibble","title":"take_while","content":"pub fn take_while(\n  predicate: fn(a) -&gt; Bool,\n) -&gt; Parser(List(a), a, b)\n\n\n 💡 This parser can succeed without consuming any input (if the predicate\n immediately fails). You can end up with an infinite loop if you&#39;re not\n careful. Use [`take_while1`](#take_while1) if you want to guarantee you\n take at least one token.\n\nSynonyms:\nnibble.take_while\nnibble take_while","url":"nibble.html#take_while"},{"doc":"nibble","title":"take_while1","content":"pub fn take_while1(\n  expecting: String,\n  predicate: fn(a) -&gt; Bool,\n) -&gt; Parser(List(a), a, b)\n\n\n 💡 If this parser succeeds, the list produced is guaranteed to be non-empty.\n Feel free to `let assert` the result!\n\nSynonyms:\nnibble.take_while1\nnibble take_while1","url":"nibble.html#take_while1"},{"doc":"nibble","title":"then","content":"pub fn then(\n  parser: Parser(a, b, c),\n  f: fn(a) -&gt; Parser(d, b, c),\n) -&gt; Parser(d, b, c)\n\n\nSynonyms:\nnibble.then\nnibble then","url":"nibble.html#then"},{"doc":"nibble","title":"throw","content":"pub fn throw(message: String) -&gt; Parser(a, b, c)\n The opposite of [`return`](#return), this parser always fails with the given\n message. Sometimes called [`fail`](#fail) instead.\n\nSynonyms:\nnibble.throw\nnibble throw","url":"nibble.html#throw"},{"doc":"nibble","title":"token","content":"pub fn token(tok: a) -&gt; Parser(Nil, a, b)\n\n\nSynonyms:\nnibble.token\nnibble token","url":"nibble.html#token"},{"doc":"nibble","title":"nibble","content":"\n &lt;script&gt;document.addEventListener(&#39;DOMContentLoaded&#39;, () =&gt; {\n  // The following is a naughty script that re-arranges the documentation at\n  // runtime. Gleam emits functions alphabetically but we prefer a different\n  // approach that guides the reader through the API from top to bottom.\n  //\n  // If JavaScript is disabled then this obviously doesn&#39;t run and folks get\n  // the default alphabetical order instead.\n\n  const orderTypes = [ &#39;Parser&#39;, &#39;Error&#39;, &#39;DeadEnd&#39;, &#39;Loop&#39; ]\n  const orderFunctions = [\n    { names: [&#39;run&#39;, &#39;return&#39;, &#39;succeed&#39;, &#39;fail&#39;, &#39;throw&#39;] },\n    { heading: &#39;Basics&#39;, names: [&#39;token&#39;, &#39;any&#39;, &#39;eof&#39;, &#39;lazy&#39;] },\n    { heading: &#39;Pipelines&#39;, names: [&#39;succeed&#39;, &#39;fail&#39;, &#39;then&#39;, &#39;dict&#39;, &#39;replace&#39;, &#39;in&#39;] },\n    { heading: &#39;With &lt;code&gt;use&lt;/code&gt;&#39;, names: [&#39;return&#39;, &#39;throw&#39;, &#39;do&#39;, &#39;do_in&#39;] },\n    { heading: &#39;Predicates&#39;, names: [&#39;take_if&#39;, &#39;take_while&#39;, &#39;take_while1&#39;, &#39;take_until&#39;, &#39;take_until1&#39;, &#39;take_map&#39;, &#39;take_map_while&#39;, &#39;take_map_while1&#39;] },\n    { heading: &#39;Transforms&#39;, names: [&#39;take_map&#39;, &#39;take_map_while&#39;, &#39;take_map_while1&#39;] },\n    { heading: &#39;Branching&#39;, names: [&#39;one_of&#39;, &#39;optional&#39;, &#39;or&#39;] },\n    { heading: &#39;Looping&#39;, names: [&#39;many&#39;, &#39;many1&#39;, &#39;sequence&#39;, &#39;loop&#39;] },\n    { heading: &#39;Backtracking&#39;, names: [&#39;backtrackable&#39;, &#39;commit&#39;] },\n    { heading: &#39;Debugging&#39;, names: [&#39;inspect&#39;] }\n  ]\n\n  const sidebarFunctionsList =\n    [...document.querySelectorAll(&#39;h2&#39;)]\n      .find(h2 =&gt; h2.innerText === &#39;Functions&#39;)\n      .nextElementSibling\n\n  const sidebarFunctions =\n    Array.from(sidebarFunctionsList.children)\n      .reduce((obj, li) =&gt; {\n        li.remove()\n        return { ...obj, [li.innerText]: li }\n      }, {})\n\n\n  for (const { heading, names } of orderFunctions) {\n    if (heading) {\n      sidebarFunctionsList.appendChild(\n        Object.assign(document.createElement(&#39;h3&#39;), {\n          innerHTML: heading,\n        })\n      )\n    }\n\n    for (const name of names) {\n      sidebarFunctionsList.appendChild(sidebarFunctions[name])\n    }\n  }\n\n  const docsFunctionsList = document.querySelector(&#39;section:has(h1#module-functions)&#39;)\n  const docsFunctions =\n    Array.from(docsFunctionsList.querySelectorAll(&#39;div.member&#39;))\n      .reduce((obj, div) =&gt; {\n        div.remove()\n        return { ...obj, [div.querySelector(&#39;div.member-name h2&#39;).innerText.trim()]: div }\n      }, {})\n\n  for (const { heading, names } of orderFunctions) {\n    if (heading) {\n      docsFunctionsList.appendChild(\n        Object.assign(document.createElement(&#39;h3&#39;), {\n          className: &#39;module-member-kind&#39;,\n          innerHTML: heading,\n        })\n      )\n    }\n\n    for (const name of names) {\n      docsFunctionsList.appendChild(docsFunctions[name])\n    }\n  }\n })&lt;/script&gt;\n","url":"nibble.html"},{"doc":"nibble/pratt","title":"Config","content":"pub opaque type Config(a, tok, ctx)\n\n\nSynonyms:\nnibble/pratt.Config\nnibble/pratt Config","url":"nibble/pratt.html#Config"},{"doc":"nibble/pratt","title":"Operator","content":"pub opaque type Operator(a, tok, ctx)\n\n\nSynonyms:\nnibble/pratt.Operator\nnibble/pratt Operator","url":"nibble/pratt.html#Operator"},{"doc":"nibble/pratt","title":"expression","content":"pub fn expression(\n  one_of first: List(fn(Config(a, b, c)) -&gt; Parser(a, b, c)),\n  and_then then: List(Operator(a, b, c)),\n  dropping spaces: Parser(Nil, b, c),\n) -&gt; Parser(a, b, c)\n\nSynonyms:\nnibble/pratt.expression\nnibble/pratt expression","url":"nibble/pratt.html#expression"},{"doc":"nibble/pratt","title":"infix_left","content":"pub fn infix_left(\n  precedence: Int,\n  operator: Parser(Nil, a, b),\n  apply: fn(c, c) -&gt; c,\n) -&gt; Operator(c, a, b)\n\nSynonyms:\nnibble/pratt.infix_left\nnibble/pratt infix_left","url":"nibble/pratt.html#infix_left"},{"doc":"nibble/pratt","title":"infix_right","content":"pub fn infix_right(\n  precedence: Int,\n  operator: Parser(Nil, a, b),\n  apply: fn(c, c) -&gt; c,\n) -&gt; Operator(c, a, b)\n\nSynonyms:\nnibble/pratt.infix_right\nnibble/pratt infix_right","url":"nibble/pratt.html#infix_right"},{"doc":"nibble/pratt","title":"postfix","content":"pub fn postfix(\n  precedence: Int,\n  operator: Parser(Nil, a, b),\n  apply: fn(c) -&gt; c,\n) -&gt; Operator(c, a, b)\n\nSynonyms:\nnibble/pratt.postfix\nnibble/pratt postfix","url":"nibble/pratt.html#postfix"},{"doc":"nibble/pratt","title":"prefix","content":"pub fn prefix(\n  precedence: Int,\n  operator: Parser(Nil, a, b),\n  apply: fn(c) -&gt; c,\n) -&gt; fn(Config(c, a, b)) -&gt; Parser(c, a, b)\n\nSynonyms:\nnibble/pratt.prefix\nnibble/pratt prefix","url":"nibble/pratt.html#prefix"},{"doc":"nibble/pratt","title":"sub_expression","content":"pub fn sub_expression(\n  config: Config(a, b, c),\n  precedence: Int,\n) -&gt; Parser(a, b, c)\n\nSynonyms:\nnibble/pratt.sub_expression\nnibble/pratt sub_expression","url":"nibble/pratt.html#sub_expression"},{"doc":"nibble/pratt","title":"nibble/pratt","content":"","url":"nibble/pratt.html"},{"doc":"nibble/predicates","title":"is_digit","content":"pub fn is_digit(grapheme: String) -&gt; Bool\n\nSynonyms:\nnibble/predicates.is_digit\nnibble/predicates is_digit","url":"nibble/predicates.html#is_digit"},{"doc":"nibble/predicates","title":"is_lower_ascii","content":"pub fn is_lower_ascii(grapheme: String) -&gt; Bool\n\nSynonyms:\nnibble/predicates.is_lower_ascii\nnibble/predicates is_lower_ascii","url":"nibble/predicates.html#is_lower_ascii"},{"doc":"nibble/predicates","title":"is_upper_ascii","content":"pub fn is_upper_ascii(grapheme: String) -&gt; Bool\n\nSynonyms:\nnibble/predicates.is_upper_ascii\nnibble/predicates is_upper_ascii","url":"nibble/predicates.html#is_upper_ascii"},{"doc":"nibble/predicates","title":"is_whitespace","content":"pub fn is_whitespace(grapheme: String) -&gt; Bool\n\nSynonyms:\nnibble/predicates.is_whitespace\nnibble/predicates is_whitespace","url":"nibble/predicates.html#is_whitespace"},{"doc":"nibble/predicates","title":"string","content":"pub fn string(str: String, predicate: fn(String) -&gt; Bool) -&gt; Bool\n\nSynonyms:\nnibble/predicates.string\nnibble/predicates string","url":"nibble/predicates.html#string"},{"doc":"nibble/predicates","title":"nibble/predicates","content":"","url":"nibble/predicates.html"}]);